@@ -2,8 +2,8 @@
 ##' with the counting process.
 ##' @name PEjmcs
 ##' @aliases PEjmcs
-##' @param object object of class 'jmcs'.
 ##' @param seed a numeric value of seed to be specified for cross validation.
+##' @param object object of class 'jmcs'.
 ##' @param landmark.time a numeric value of time for which dynamic prediction starts..
 ##' @param horizon.time a numeric vector of future times for which predicted probabilities are to be computed.
 ##' @param obs.time a character string of specifying a longitudinal time variable.
@@ -14,6 +14,12 @@
 ##' function will perform. Default is 10000.
 ##' @param n.cv number of folds for cross validation. Default is 3.
 ##' @param survinitial Fit a Cox model to obtain initial values of the parameter estimates. Default is TRUE.
+##' @param initial.para Initial guess of parameters for cross validation. Default is FALSE.
+##' @param LOCF a logical value to indicate whether the last-observation-carried-forward approach applies to prediction. 
+##' If \code{TRUE}, then \code{LOCFcovariate} and \code{clongdata} must be specified to indicate 
+##' which time-dependent survival covariates are included for dynamic prediction. Default is FALSE.
+##' @param LOCFcovariate a vector of string with time-dependent survival covariates if \code{LOCF = TRUE}. Default is NULL.
+##' @param clongdata a long format data frame where time-dependent survival covariates are incorporated. Default is NULL.
 ##' @param ... Further arguments passed to or from other methods.
 ##' @return a list of matrices with conditional probabilities for subjects.
 ##' @author Shanpeng Li \email{lishanpeng0913@ucla.edu}
@@ -21,10 +27,12 @@
 ##' @export
 ##' 
 
-PEjmcs <- function(object, seed = 100, landmark.time = NULL, horizon.time = NULL, 
-                      obs.time = NULL, method = c("Laplace", "GH"), 
-                      quadpoint = NULL, maxiter = NULL, n.cv = 3, 
-                   survinitial = TRUE, ...) {
+PEjmcs <- function(seed = 100, object, landmark.time = NULL, horizon.time = NULL, 
+                   obs.time = NULL, method = c("Laplace", "GH"), 
+                   quadpoint = NULL, maxiter = NULL, n.cv = 3, 
+                   survinitial = TRUE, 
+                   initial.para = FALSE,
+                   LOCF = FALSE, LOCFcovariate = NULL, clongdata = NULL,...) {
   
   if (!inherits(object, "jmcs"))
     stop("Use only with 'jmcs' xs.\n")
@@ -60,6 +68,18 @@ PEjmcs <- function(object, seed = 100, landmark.time = NULL, horizon.time = NULL
   random <- all.vars(object$random) 
   ID <- random[length(random)]
   
+  if (initial.para) {
+    initial.para <- list(beta = object$beta,
+                         sigma = object$sigma, 
+                         gamma1 = object$gamma1,
+                         gamma2 = object$gamma2,
+                         alpha1 = object$nu1,
+                         alpha2 = object$nu2,
+                         Sig = object$Sig)
+  } else {
+    initial.para <- NULL
+  }
+  
   folds <- caret::groupKFold(c(1:nrow(cdata)), k = n.cv)
   Brier.cv <- list()
   MAE.cv <- list()
@@ -72,7 +92,9 @@ PEjmcs <- function(object, seed = 100, landmark.time = NULL, horizon.time = NULL
                       long.formula = long.formula,
                       surv.formula = surv.formula,
                       quadpoint = quadpoint, random = object$random, 
-                      survinitial = survinitial), silent = TRUE)
+                      survinitial = survinitial,
+                      opt = object$opt,
+                      initial.para = initial.para), silent = TRUE)
     
     if ('try-error' %in% class(fit)) {
       writeLines(paste0("Error occured in the ", t, " th training!"))
@@ -94,10 +116,18 @@ PEjmcs <- function(object, seed = 100, landmark.time = NULL, horizon.time = NULL
       NewyID <- unique(val.ydata[, ID])
       val.cdata <- val.cdata[val.cdata[, ID] %in% NewyID, ]
       
+      if (LOCF) {
+        val.clongdata <- clongdata[clongdata[, ID] %in% val.cdata[, ID], ]
+      } else {
+        val.clongdata <- NULL
+      }
+      
       survfit <- try(survfitjmcs(fit, ynewdata = val.ydata, cnewdata = val.cdata, 
-                                   u = horizon.time, method = method, 
-                                   Last.time = rep(landmark.time, nrow(val.cdata)),
-                                   obs.time = obs.time, quadpoint = quadpoint), silent = TRUE)
+                                 u = horizon.time, method = method, 
+                                 Last.time = landmark.time,
+                                 obs.time = obs.time, quadpoint = quadpoint,
+                                 LOCF = LOCF, LOCFcovariate = LOCFcovariate, 
+                                 clongdata = val.clongdata), silent = TRUE)
       
       if ('try-error' %in% class(survfit)) {
         writeLines(paste0("Error occured in the ", t, " th validation!"))
