@@ -2,7 +2,7 @@ logLikCRmv <- function(data, b) {
   
   total <- 0
   index <- 1
-  numBio <- length(data$Y)
+  numBio <- length(data$YList)
   
   
   sum.alpha1i <- 0
