@@ -13,7 +13,7 @@ double getloglikeCpseudo(const Eigen::VectorXd & beta,
                            const Eigen::VectorXd & Y, const Eigen::MatrixXd & X2, 
                            const Eigen::VectorXd & survtime, 
                            const Eigen::VectorXd & cmprsk, const Eigen::VectorXd & mdata, 
-                           const Eigen::VectorXd & mdataS, const Eigen::MatrixXd & xsmatrix, 
+                           const Eigen::VectorXi & mdataS, const Eigen::MatrixXd & xsmatrix, 
                            const Eigen::MatrixXd & wsmatrix, 
                            const Eigen::VectorXd & CUH01, 
                            const Eigen::VectorXd & CUH02, 
